<?php

register_activation_hook( __FILE__, 'setup' );
add_action( "wp_loaded", 'onelogin_register_files' );
add_action( 'wp_authenticate', 'onelogin_client_login_hook' );
add_filter( 'template_redirect', 'auto_sso_init', 11 );
add_action( 'login_form', 'wp_sso_login_form_button' );
add_shortcode( 'sso_button', 'single_sign_on_login_button_shortcode' );
add_filter( 'login_message', 'wpo_login_only_message' );
add_action( 'admin_init', 'admin_init' );
add_action( 'admin_menu', 'add_page' );
add_filter( 'rewrite_rules_array', 'create_rewrite_rules' );
add_filter( 'query_vars', 'add_query_vars' );
add_filter( 'wp_loaded', 'dws_flush_rewrite_rules' );
add_action( 'template_redirect', 'template_redirect_intercept' );

function setup() {

    $default_settings = array(
        'client_id'            => '',
        'client_secret'        => '',
        'server_url'           => '',
        'server_oauth_trigger' => 'oauth',
        'server_auth_endpoint' => 'authorize',
        'server_token_endpont' => 'token',
        'server_user_endpoint' => 'me'
    );

    $options = get_option( "wposso_options" );
    if ( ! isset( $options["server_url"] ) ) {
        update_option( "wposso_options", $default_settings );
    }
}

function onelogin_register_files() {
	wp_enqueue_style( 'dws_oauth_admin', plugins_url( '/assets/css/admin.css', __FILE__ ) );
	wp_enqueue_script( 'dws_oauth_admin', plugins_url( '/assets/js/admin.js', __FILE__ ) );
}

function onelogin_client_login_hook( $user ) {

	if ( ! empty( $_POST['log'] ) && ! empty( $_POST['pwd'] ) && isset( $_POST['wp-submit'] ) && $_POST['wp-submit'] == 'Log In' ) {

		/*
		 * Get the One Client Configuration
		 */
		$options = get_option( "wposso_options" );

		// Start the One Login Flow here using the credentials of the user.
		$curl_post_data = array(
			'grant_type' => 'password',
			'username'   => sanitize_text_field( $_POST['log'] ),
			'password'   => $_POST['pwd'],
			//'client_id'     => $client_id,
			//'client_secret' => $client_secret
		);

		$curl = curl_init( $options['server_url'] . '?oauth=token' );
		curl_setopt( $curl, CURLOPT_USERPWD, $options['client_id'] . ':' . $options['client_secret'] );
		curl_setopt( $curl, CURLOPT_RETURNTRANSFER, true );
		curl_setopt( $curl, CURLOPT_POST, true );
		curl_setopt( $curl, CURLOPT_POSTFIELDS, $curl_post_data );
		curl_setopt( $curl, CURLOPT_SSL_VERIFYPEER, false );
		curl_setopt( $curl, CURLOPT_USERAGENT, 'Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.1.2) Gecko/20090729 Firefox/3.5.2 GTB5' );
		curl_setopt( $curl, CURLOPT_REFERER, site_url() );

		$curl_response = curl_exec( $curl );
		curl_close( $curl );

		$response = json_decode( $curl_response );

		// If the access token is not empty, the login was valid
		if ( ! empty( $response->access_token ) ) {
			$user_info_request = wp_remote_get( $options['server_url'] . '?oauth=me&access_token=' . $response->access_token, array(
				'timeout'     => 120,
				'httpversion' => '1.1',
				'sslverify'   => false
			) );

			$user_info = json_decode( wp_remote_retrieve_body( $user_info_request ) );

			$user_id = username_exists( $user_info->user_login );

			if ( ! $user_id && email_exists( $user_info->user_email ) == false ) {

				// Does not have an account... Register and then log the user in
				$random_password = wp_generate_password( $length = 12, $include_standard_special_chars = false );
                $userdata = array(
                    'user_login'            => $user_info->user_login,
                    'user_pass'             => $random_password,
                    'user_email'            => $user_info->user_email,
                );
                
                $user_id         = wp_insert_user( $userdata );

				if ( isset( $user_info->first_name ) ) {
					update_user_meta( $user_id, 'first_name', $user_info->first_name );
				}

				if ( isset( $user_info->last_name ) ) {
					update_user_meta( $user_id, 'last_name', $user_info->last_name );
				}

				do_action( 'onelogin_user_created', $user_info, 1 );

				wp_clear_auth_cookie();
				wp_set_current_user( $user_id );
				wp_set_auth_cookie( $user_id );

				if ( is_user_logged_in() ) {
					wp_safe_redirect( site_url() );
					exit;
				}

			} else {

				// Already Registered... Log the User In
				$random_password = __( 'User already exists.  Password inherited.' );
				$user            = get_user_by( 'login', $user_info->user_login );

				/*
		        * Added just in case the user is not used but the email may be. If the user returns false from the user ID,
		        * we should check the user by email. This may be the case when the users are preregistered outside of OAuth
		        */
				if ( ! $user ) {
					$user = get_user_by( 'email', $user_info->user_email );
				}

				if ( isset( $user_info->first_name ) ) {
					update_user_meta( $user->ID, 'first_name', $user_info->first_name );
				}

				if ( isset( $user_info->last_name ) ) {
					update_user_meta( $user->ID, 'last_name', $user_info->last_name );
				}

				// User ID 1 is not allowed
				if ( '1' === $user->ID ) {
					wp_die( 'For security reasons, this user can not use Digital Wordings Login' );
				}

				do_action( 'onelogin_user_login', $user_info, 1 );

				wp_clear_auth_cookie();
				wp_set_current_user( $user->ID );
				wp_set_auth_cookie( $user->ID );

				if ( is_user_logged_in() ) {
					wp_safe_redirect( site_url() );
					exit;
				}

			}
		}

		exit();
	}
}

function auto_sso_init( $template ) {

	// If the user is not logged in, load in the SSO client in a child IFrame instead of redirect
	if ( ! is_user_logged_in() ) {
		$options = get_option( "wposso_options" );
		if ( isset( $options["auto_sso"] ) && $options["auto_sso"] == 1 ) {

			/*
			 * @todo Build options instead of just a last query. This will be able to handle the pass of the iframe
			 */
			global $wp;
			$last_page = home_url( $wp->request );

			$params   = array(
				'oauth'         => 'authorize',
				'response_type' => 'code',
				'client_id'     => $options['client_id'],
				'client_secret' => $options['client_secret'],
				'redirect_uri'  => site_url( '?auth=sso' ),
				'state'         => urlencode( $last_page ),
			);
			$redirect = add_query_arg( $params, $options['server_url'] );

			wp_redirect( $redirect );
			exit;
		}
	}
}

function wp_sso_login_form_button() {
	?>
    <a style="color:#FFF; width:100%; text-align:center; margin-bottom:1em;" class="button button-primary button-large" href="<?php echo site_url( '?auth=sso' ); ?>">Digital Wordings Login</a>
    <div style="clear:both;"></div>
	<?php
}

function single_sign_on_login_button_shortcode( $atts ) {
	$a = shortcode_atts( array(
		'type'   => 'primary',
		'title'  => 'Login using Digital Wordings Login',
		'class'  => 'sso-button',
		'target' => '_blank',
		'text'   => 'Digital Wordings Login'
	), $atts );

	return '<a class="' . $a['class'] . '" href="' . site_url( '?auth=sso' ) . '" title="' . $a['title'] . '" target="' . $a['target'] . '">' . $a['text'] . '</a>';
}

function wpo_login_only_message( $message ) {
	if ( empty( $message ) && isset( $_GET['wpo_login_only'] ) ) {
		return '<p><strong>' . apply_filters( 'wpo_login_only_msg', 'You need to have an account to use Digital Wordings Login' ) . '</strong></p>';
	} else {
		return $message;
	}
}

function admin_init() {
    $option_name = 'wposso_options';
    register_setting( 'wposso_options', $option_name, 'validate' );
}

function validate( $input ) {
    $input['redirect_to_dashboard'] = isset( $input['redirect_to_dashboard'] ) ? $input['redirect_to_dashboard'] : 0;
    $input['login_only'] = isset( $input['login_only'] ) ? $input['login_only'] : 0;

    return $input;
}

function add_page() {
    add_options_page( 'OAuth 2.0 SSO', 'OAuth 2.0 SSO', 'manage_options', 'onelogin_client_settings', 'options_do_page' );
}

function options_do_page() {
    $option_name = 'wposso_options';
    $options = get_option( $option_name );
    admin_head();
    ?>
    <div class="wrap">
        <h2>Digital Wordings login Configuration</h2>
        <p>This plugin is meant to be used with <a href="https://wp-oauth.com">WordPress OAuth Server</a>.</p>
        <p>
            When activated, this plugin adds a Digital Wordings login button to the login screen.
            <br/>
            <strong>NOTE:</strong> If you want to add a
            custom link anywhere in your theme simply link to
            <strong><?php echo site_url( '?auth=sso' ); ?></strong>
            if the user is not logged in.
        </p>
        <div id="accordion">
            <h4>Step 1: Setup</h4>
            <div>
                <strong>Setting up WordPress OAuth Server</strong>
                <ol>
                    <li>Enable Authorization Code</li>
                    <li>Create a new client and set the Redirect URI to:
                        <strong class="code"><?php echo site_url( '?auth=sso' ); ?></strong></li>
                    <li>Copy the Client ID and Client Secret in Step 2 below.</li>
                </ol>
            </div>
            <h4 id="ssso-configuration">Step 2: Configuration</h4>
            <div>
                <form method="post" action="options.php">
                    <?php settings_fields( 'wposso_options' ); ?>
                    <table class="form-table">
                        <tr valign="top">
                            <th scope="row">Client ID</th>
                            <td>
                                <input type="text" name="<?php echo $option_name ?>[client_id]" min="10"
                                       value="<?php echo $options["client_id"]; ?>"/>
                            </td>
                        </tr>

                        <tr valign="top">
                            <th scope="row">Client Secret</th>
                            <td>
                                <input type="text" name="<?php echo $option_name ?>[client_secret]" min="10"
                                       value="<?php echo $options["client_secret"]; ?>"/>
                            </td>
                        </tr>

                        <tr valign="top">
                            <th scope="row">OAuth Server URL</th>
                            <td>
                                <input type="text" name="<?php echo $option_name ?>[server_url]" min="10"
                                       value="<?php echo $options["server_url"]; ?>"/>
                                <p class="description">Example: https://your-site.com</p>
                            </td>
                        </tr>

                        <tr valign="top">
                            <th scope="row">Redirect to the dashboard after signing in</th>
                            <td>
                                <input type="checkbox"
                                       name="<?php echo $option_name ?>[redirect_to_dashboard]"
                                       value="1" <?php echo @$options['redirect_to_dashboard'] == 1 ? 'checked="checked"' : ''; ?> />
                            </td>
                        </tr>
                        <tr valign="top">
                            <th scope="row">Restrict flow to log in only</th>
                            <td>
                                <input type="checkbox"
                                       name="<?php echo $option_name ?>[login_only]"
                                       value="1" <?php echo @$options['login_only'] == 1 ? 'checked="checked"' : ''; ?> />
                            </td>
                        </tr>
                    </table>

                    <p class="submit">
                        <input type="submit" class="button-primary" value="<?php _e( 'Save Changes' ) ?>"/>
                    </p>
            </div>

            </form>
        </div>
    </div>
    <div style="clear:both;"></div>
    </div>
    <?php
}

function admin_head() {
    wp_enqueue_script( 'jquery-ui-accordion' );
    wp_enqueue_style( 'wpsso_admin' );
    wp_enqueue_script( 'wpsso_admin' );
}

function create_rewrite_rules( $rules ) {
    global $wp_rewrite;
    $newRule  = array( 'auth/(.+)' => 'index.php?auth=' . $wp_rewrite->preg_index( 1 ) );
    $newRules = $newRule + $rules;
    return $newRules;
}

function add_query_vars( $qvars ) {
    $qvars[] = 'auth';

    return $qvars;
}

function dws_flush_rewrite_rules() {
    global $wp_rewrite;
    $wp_rewrite->flush_rules();
}

function template_redirect_intercept() {
    global $wp_query;
    if ( $wp_query->get( 'auth' ) && $wp_query->get( 'auth' ) == 'sso' ) {
        require_once plugin_dir_path( __FILE__ ).'oauth-callback.php';
        exit;
    }
}