<?php

add_action( 'wp_authenticate_user', 'dws_wp_login');
add_action( 'init', 'remove_actions');
add_action( 'wpoc_user_created', 'dws_custom_user_created' );
add_action( 'wpoc_user_login', 'dws_user_login' );

function dws_wp_login($user) {
    if( isset($user->data->ID) && !empty($user->data->ID) ) {
        $is_dws_login = get_user_meta($user->data->ID, 'login_through_dw', true);
        if( $is_dws_login ) {
            return new WP_Error(
                'login_via_dws',
                sprintf(
                    /* translators: %s: Email address. */
                    __( '<strong>Error</strong>: You need to login via Digital Wordings Login' ),
                    '<strong>' . $email . '</strong>'
                )
            );
        }
    }
    return $user;
}

add_action('rest_api_init', function () {
    register_rest_route( 'digitalwordings/v1', 'credentials',array(
                  'methods'  => 'POST',
                  'callback' => 'get_credentials_from_masters',
                  'args' => array(
                      'id' => array(
                          'validate_callback' => function( $param, $request, $key ) {
                              return is_numeric( $param );
                          }
                      ),
                  ),
        ));
  });

function get_credentials_from_masters( $data ){
    $dws_nonce      =   $data['dws_nonce'];
    $post_id        =   $data['post_id'];
    $client_id      =   $data['client_id'];
    $client_secret  =   $data['client_secret'];
    $redirect_uri   =   $data['redirect_uri'];
    $server_url     =   $data['server_url'];
    
    $args = array(
        'dws_nonce' => $dws_nonce,
        'post_id' => $post_id
    );
    
    $api_server     =   'http://accounts.digitalwordings.com/';
    // $api_server     =   'http://localhost/eight_medi_theme/';
    $api_route      =   'wp-json/digitalwordings/v1/credentials_verify/';
    $url            =   $api_server . $api_route;
    $response       =   wp_remote_post( $url, $args );
    $body           =   wp_remote_retrieve_body( $response );
    $body           =   json_decode( $body, true );

    if( $body['verified'] == true ) {
       $wposso_options['client_id'] = $data['client_id'];
       $wposso_options['client_secret'] = $data['client_secret'];
       $wposso_options['server_url'] = $data['server_url'];
       $wposso_options['post_id'] = $data['post_id'];
       update_option('wposso_options', $wposso_options);
    }

}

function remove_actions() {
    remove_action( 'wp_authenticate', 'onelogin_client_login_hook' );
    remove_action( 'admin_menu', 'add_page' );
    remove_shortcode( 'sso_button' );
}

function dws_custom_user_created( $user_info ) {

    $user_role = dws_receive_role( $user_info->ID );

    $user = get_user_by( 'login', $user_info->user_login );
    if ( ! $user ) {
        $user = get_user_by( 'email', $user_info->user_email );
    }

    $dws_userdata = array(
        'ID'            => $user->ID,
        'role'          => $user_role,
    );
    
    wp_update_user( $dws_userdata );
    update_user_meta( $user_id, 'login_through_dw', true );
}

function dws_user_login( $user_info ) {

    if( isset( $user_info->ID ) ) {

        $user_role = dws_receive_role( $user_info->ID );

        $user = get_user_by( 'login', $user_info->user_login );
        if ( ! $user ) {
            $user = get_user_by( 'email', $user_info->user_email );
        }
        $dws_userdata = array(
            'ID'            => $user->ID,
            'role'          => $user_role,
        );
        $u_id = wp_update_user( $dws_userdata );
    }
}

function dws_receive_role( $id ) {

    if( isset( $_REQUEST['state'] ) && !empty( $_REQUEST['state'] ) ) {
        $client_web = $_REQUEST['state'];
        if (strpos( $_REQUEST['state'], 'wp-admin') !== false) {
            $client_web_total = explode("wp-admin", $_REQUEST['state']);
            $client_web = $client_web_total[0];
        }
        $client_web_last_dgt = substr($client_web, -1);
        if( $client_web_last_dgt == '/' ) {
            $client_web = $client_web;
        } else {
            $client_web = $client_web.'/';
        }
        $client_web = $client_web.'?auth=sso';
        $role_args 	= array(
            'method' 	=> 'POST',
            'body'      => array(
                'user_id' => $id,
                'data' => 'verified',
                'client_web' => isset( $client_web ) ? $client_web : ''
            )
        );
        
        $api_server     =   'http://accounts.digitalwordings.com/';
        // $api_server     =   'http://localhost/eight_medi_theme/';
        $api_route      =   'wp-json/digitalwordings/v1/role_update/';
        $role_url       =   $api_server . $api_route;
        $response       =   wp_remote_post( $role_url, $role_args );
        $body           =   wp_remote_retrieve_body( $response );
        $body           =   json_decode( $body, true );
        
        if( isset( $body['external_role'] ) && !empty( $body['external_role'] ) ) {
            $user_role = $body['external_role'];
            return $user_role;
        }
    }

}