<?php

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

if ( ! class_exists( 'Dws_Templates_Manager' ) ) {

	/**
	 * Define Jet_Elements_Templates_Manager class
	 */
	class Dws_Templates_Manager {

		/**
		 * A reference to an instance of this class.
		 *
		 * @since 1.0.0
		 * @var   object
		 */
		private static $instance = null;

		/**
		 * Categories option name
		 * @var string
		 */
		protected $option = 'dws_elements_categories';

		/**
		 * Templates option name
		 * @var string
		 */
		protected $templates_option = 'dws_elements_templates';


		public function __construct() {

			// Register jet-templates source
			add_action( 'elementor/init', array( $this, 'register_templates_source' ) );
			
			add_action( 'elementor/editor/before_enqueue_scripts',[$this, 'dws_hide_pro_notice']);
			add_action( 'pre_update_option_elementor_remote_info_library', [$this, 'dws_hide_pro_templates'], 1000, 3 );

			if ( defined( 'Elementor\Api::LIBRARY_OPTION_KEY' ) ) {
				// Add JetImpex templates to Elementor templates list
				add_filter( 'option_' . Elementor\Api::LIBRARY_OPTION_KEY, array( $this, 'prepend_categories' ) );
			}

			// Process JetImpext template request
			if ( defined( 'ELEMENTOR_VERSION' ) && version_compare( ELEMENTOR_VERSION, '2.2.8', '>' ) ) {
				add_action( 'elementor/ajax/register_actions', array( $this, 'register_ajax_actions' ), 20 );
			} else {
				add_action( 'wp_ajax_elementor_get_template_data', array( $this, 'force_dws_template_source' ), 0 );
			}

			if ( defined( 'ELEMENTOR_VERSION' )
				 && version_compare( ELEMENTOR_VERSION, '3.5.0', '>=' )
				 && version_compare( ELEMENTOR_VERSION, '3.10.0', '<' )
			) {
				add_filter( 'option_' . Elementor\Api::LIBRARY_OPTION_KEY, array( $this, 'prepend_templates' ) );
			}

			if ( defined( 'ELEMENTOR_VERSION' ) && version_compare( ELEMENTOR_VERSION, '3.10.0', '>=' ) ) {
				$transient_key = Elementor\TemplateLibrary\Source_Remote::TEMPLATES_DATA_TRANSIENT_KEY_PREFIX . ELEMENTOR_VERSION;
				add_filter( 'transient_' . $transient_key, array( $this, 'prepend_remote_templates' ) );
			}
		}
		
		public function dws_hide_pro_templates( $value ) {
				if ( ! empty( $value['templates'] ) ) {
				$normal_templates = wp_list_filter( $value['templates'], array( 'author' => 'Digital Wordings' ), 'AND' );
				$value['templates'] = $normal_templates;
					if ( ! empty( $normal_templates ) ) {
						$value['templates'] = $normal_templates;
					}
				}
				return $value;
		}
		
		public function dws_hide_pro_notice() {
			if ( ! is_user_logged_in() ) {
				return;
			}
	        
	        echo '<style id="elementor-wl-branding">';
			?>
			#elementor-notice-bar {
			display: none !important;
		}
		<?php	echo '</style>';
		}
		/**
		 * Register
		 *
		 * @return [type] [description]
		 */
		public function register_templates_source() {

			require DWS_DIR . '/template-library-class.php';

			$elementor = Elementor\Plugin::instance();
			$elementor->templates_manager->register_source( 'Dws_Templates_Source' );

		}

		/**
		 * Return transient key
		 *
		 * @return [type] [description]
		 */
		public function transient_key() {
			return $this->option . '_' . 43;
		}

		public function templates_transient_key() {
			return $this->templates_option . '_' . 43;
		}

		/**
		 * Retrieves categories list
		 *
		 * @return [type] [description]
		 */
		public function get_categories() {

			$categories = get_transient( $this->transient_key() );

			if ( ! $categories ) {
				$categories = $this->prepend_categories();
				set_transient( $this->transient_key(), $categories, WEEK_IN_SECONDS );
			}

			return $categories;
		}

		/**
		 * Get categories from JetElements API
		 *
		 * @return array
		 */
		public function remote_get_categories() {
            $api_server     = 'https://master.digitalwording.co.in/';
		    $api_route      = 'wp-json/digitalwordings/v1';
			$url      = $api_server . $api_route . '/categories/';
			$response = wp_remote_get( $url, array( 'timeout' => 60 ) );
			$body     = wp_remote_retrieve_body( $response );
			$body     = json_decode( $body, true );
			return ! empty( $body['data'] ) ? $body['data'] : array();
		}

		/**
		 * Add JetImpex templates to Elementor templates list
		 *
		 * @param  [type] $templates [description]
		 * @return [type]            [description]
		 */
		public function prepend_categories( $library_data ) {

			$categories = $this->remote_get_categories();

			if ( ! empty( $categories ) ) {

				if ( defined( 'ELEMENTOR_VERSION' ) && version_compare( ELEMENTOR_VERSION, '2.3.9', '>' ) ) {
					$library_data['types_data']['block']['categories'] = array_merge( $categories, $library_data['types_data']['block']['categories'] );
				} else {
					$library_data['categories'] = array_merge( $categories, $library_data['categories'] );
				}

				return $library_data;

			} else {
				return $library_data;
			}

		}

		public function prepend_templates( $library_data ) {

			$templates = $this->get_templates();

			if ( ! empty( $templates ) ) {
				$library_data['templates'] = array_merge( $library_data['templates'], $templates );
			}

			return $library_data;
		}

		public function prepend_remote_templates( $remote_templates ) {

			$templates = $this->get_templates();

			if ( ! empty( $templates ) ) {
				$remote_templates = array_merge( $remote_templates, $templates );
			}

			return $templates;
		}

		public function get_templates() {

				$source = Elementor\Plugin::instance()->templates_manager->get_source( 'dws-templates' );
				
				if ( $source ) {
					$templates = $source->get_items();

					if ( ! empty( $templates ) ) {

						$templates = array_map( function ( $template ) {

							$template['id'] = $template['template_id'];
							$template['tmpl_created'] = $template['date'];
							$template['tags'] = json_encode( $template['tags'] );
							$template['is_pro'] = $template['isPro'];
							$template['access_level'] = $template['accessLevel'];
							$template['popularity_index'] = $template['popularityIndex'];
							$template['trend_index'] = $template['trendIndex'];
							$template['has_page_settings'] = $template['hasPageSettings'];

							unset( $template['template_id'] );
							unset( $template['date'] );
							unset( $template['isPro'] );
							unset( $template['accessLevel'] );
							unset( $template['popularityIndex'] );
							unset( $template['trendIndex'] );
							unset( $template['hasPageSettings'] );

							return $template;
						}, $templates );

						set_transient( $this->templates_transient_key(), $templates, WEEK_IN_SECONDS );

					} else {
						$templates = array();
					}
				}

			return $templates;
		}

		/**
		 * Register AJAX actions
		 *
		 * @param $ajax
		 */
		public function register_ajax_actions( $ajax ) {
			if ( ! isset( $_REQUEST['actions'] ) ) {
				return;
			}

			$actions = json_decode( stripslashes( $_REQUEST['actions'] ), true );
			$data    = false;

			foreach ( $actions as $id => $action_data ) {
				if ( ! isset( $action_data['get_template_data'] ) ) {
					$data = $action_data;
				}
			}

			if ( ! $data ) {
				return;
			}

			if ( ! isset( $data['data'] ) ) {
				return;
			}

			$data = $data['data'];

			if ( empty( $data['template_id'] ) ) {
				return;
			}

			if ( false === strpos( $data['template_id'], 'dws_' ) ) {
				return;
			}

			$ajax->register_ajax_action( 'get_template_data', array( $this, 'get_dws_template_data' ) );
		}

		/**
		 * Get jet template data.
		 *
		 * @param $args
		 *
		 * @return mixed
		 */
		public function get_dws_template_data( $args ) {

			$source = Elementor\Plugin::instance()->templates_manager->get_source( 'dws-templates' );

			$data = $source->get_data( $args );

			return $data;
		}

		/**
		 * Return jet template data insted of elementor template.
		 *
		 * @return [type] [description]
		 */
		public function force_dws_template_source() {

			if ( empty( $_REQUEST['template_id'] ) ) {
				return;
			}

			if ( false === strpos( $_REQUEST['template_id'], 'dws_' ) ) {
				return;
			}

			$_REQUEST['source'] = 'dws-templates';

		}

		/**
		 * Returns the instance.
		 *
		 * @since  1.0.0
		 * @return object
		 */
		public static function get_instance() {

			// If the single instance hasn't been set, set it now.
			if ( null == self::$instance ) {
				self::$instance = new self;
			}
			return self::$instance;
		}
	}

}